@@ -96,7 +96,7 @@ dzl_file_transfer_finalize (GObject *object)
   DzlFileTransfer *self = (DzlFileTransfer *)object;
   DzlFileTransferPrivate *priv = dzl_file_transfer_get_instance_private (self);
 
-  dzl_clear_pointer (&priv->opers, g_ptr_array_unref);
+  g_clear_pointer (&priv->opers, g_ptr_array_unref);
 
   G_OBJECT_CLASS (dzl_file_transfer_parent_class)->finalize (object);
 }
@@ -548,7 +548,7 @@ handle_removal (DzlFileTransfer *self,
       if (oper->error != NULL)
         return;
 
-      if (g_file_query_file_type (oper->src, 0, NULL) == G_FILE_TYPE_DIRECTORY)
+      if (g_file_query_file_type (oper->src, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL) == G_FILE_TYPE_DIRECTORY)
         dzl_directory_reaper_add_directory (reaper, oper->src, 0);
 
       dzl_directory_reaper_add_file (reaper, oper->src, 0);
