@@ -368,13 +368,13 @@ dzl_graph_view_model_iter_set (DzlGraphModelIter *iter,
 
       column = g_ptr_array_index (priv->columns, column_id);
 
-      _dzl_graph_view_column_collect (column, impl->index, args);
+      _dzl_graph_view_column_collect (column, impl->index, &args);
 
       column_id = va_arg (args, gint);
     }
 
   if (column_id != -1)
-    g_critical ("Invalid column sentinal: %d", column_id);
+    g_critical ("Invalid column sentinel: %d", column_id);
 
 cleanup:
   va_end (args);
@@ -410,13 +410,13 @@ dzl_graph_view_model_iter_get (DzlGraphModelIter *iter,
 
       column = g_ptr_array_index (priv->columns, column_id);
 
-      _dzl_graph_view_column_lcopy (column, impl->index, args);
+      _dzl_graph_view_column_lcopy (column, impl->index, &args);
 
       column_id = va_arg (args, gint);
     }
 
   if (column_id != -1)
-    g_critical ("Invalid column sentinal: %d", column_id);
+    g_critical ("Invalid column sentinel: %d", column_id);
 
 cleanup:
   va_end (args);
