@@ -253,10 +253,19 @@ static void
 dzl_shortcut_controller_widget_destroy (DzlShortcutController *self,
                                         GtkWidget             *widget)
 {
+  DzlShortcutControllerPrivate *priv = dzl_shortcut_controller_get_instance_private (self);
+
   g_assert (DZL_IS_SHORTCUT_CONTROLLER (self));
   g_assert (GTK_IS_WIDGET (widget));
 
   dzl_shortcut_controller_disconnect (self);
+  g_clear_weak_pointer (&priv->widget);
+
+  if (priv->root != NULL)
+    {
+      dzl_shortcut_controller_remove (priv->root, self);
+      g_clear_object (&priv->root);
+    }
 }
 
 static void
@@ -333,7 +342,7 @@ dzl_shortcut_controller_connect (DzlShortcutController *self)
 
   manager = dzl_shortcut_controller_get_manager (self);
 
-  dzl_clear_pointer (&priv->current_chord, dzl_shortcut_chord_free);
+  g_clear_pointer (&priv->current_chord, dzl_shortcut_chord_free);
   priv->context_name = NULL;
 
   priv->widget_destroy_handler =
@@ -371,14 +380,12 @@ dzl_shortcut_controller_set_widget (DzlShortcutController *self,
       if (priv->widget != NULL)
         {
           dzl_shortcut_controller_disconnect (self);
-          g_object_remove_weak_pointer (G_OBJECT (priv->widget), (gpointer *)&priv->widget);
-          priv->widget = NULL;
+          g_clear_weak_pointer (&priv->widget);
         }
 
       if (widget != NULL && widget != priv->widget)
         {
-          priv->widget = widget;
-          g_object_add_weak_pointer (G_OBJECT (priv->widget), (gpointer *)&priv->widget);
+          g_set_weak_pointer (&priv->widget, widget);
           dzl_shortcut_controller_connect (self);
         }
 
@@ -437,8 +444,8 @@ dzl_shortcut_controller_finalize (GObject *object)
       priv->widget = NULL;
     }
 
-  dzl_clear_pointer (&priv->commands, g_hash_table_unref);
-  dzl_clear_pointer (&priv->commands_table, dzl_shortcut_chord_table_free);
+  g_clear_pointer (&priv->commands, g_hash_table_unref);
+  g_clear_pointer (&priv->commands_table, dzl_shortcut_chord_table_free);
   g_clear_object (&priv->root);
 
   while (priv->descendants.length > 0)
@@ -1221,6 +1228,6 @@ _dzl_shortcut_controller_clear (DzlShortcutController *self)
 
   g_return_if_fail (DZL_IS_SHORTCUT_CONTROLLER (self));
 
-  dzl_clear_pointer (&priv->current_chord, dzl_shortcut_chord_free);
+  g_clear_pointer (&priv->current_chord, dzl_shortcut_chord_free);
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_CURRENT_CHORD]);
 }
