@@ -812,6 +812,9 @@ dzl_shortcut_manager_run_fallbacks (DzlShortcutManager     *self,
                                     const DzlShortcutChord *chord)
 {
   DzlShortcutManagerPrivate *priv = dzl_shortcut_manager_get_instance_private (self);
+  static DzlShortcutChord *inspector_chord;
+
+  DZL_ENTRY;
 
   g_assert (DZL_IS_SHORTCUT_MANAGER (self));
   g_assert (GTK_IS_WIDGET (widget));
@@ -827,14 +830,23 @@ dzl_shortcut_manager_run_fallbacks (DzlShortcutManager     *self,
 
       dzl_shortcut_chord_get_nth_key (chord, 0, &keyval, &state);
 
+      /* Special case shift-tab, which is shown as ISO_Left_Tab when
+       * we converted into a Dazzle chord.
+       */
+      if (keyval == GDK_KEY_ISO_Left_Tab && state == 0)
+        {
+          if (gtk_bindings_activate (G_OBJECT (toplevel), keyval, GDK_SHIFT_MASK))
+            DZL_RETURN (TRUE);
+        }
+
       /* See if the toplevel activates this, like Tab, etc */
       if (gtk_bindings_activate (G_OBJECT (toplevel), keyval, state))
-        return TRUE;
+        DZL_RETURN (TRUE);
 
       /* See if there is a mnemonic active that should be activated */
       if (GTK_IS_WINDOW (toplevel) &&
           gtk_window_mnemonic_activate (GTK_WINDOW (toplevel), keyval, state))
-        return TRUE;
+        DZL_RETURN (TRUE);
 
       /*
        * See if we have something defined for this theme that
@@ -851,7 +863,24 @@ dzl_shortcut_manager_run_fallbacks (DzlShortcutManager     *self,
           dzl_g_action_name_parse_full (action, &prefix, &name, &target);
 
           if (dzl_gtk_widget_action (toplevel, prefix, name, target))
-            return TRUE;
+            DZL_RETURN (TRUE);
+        }
+
+      /*
+       * If we this is the ctrl+shift+d keybinding to activate the inspector,
+       * then try to see if we should handle that manually.
+       */
+      if G_UNLIKELY (inspector_chord == NULL)
+        inspector_chord = dzl_shortcut_chord_new_from_string ("<ctrl><shift>d");
+      if (dzl_shortcut_chord_equal (chord, inspector_chord))
+        {
+          g_autoptr(GSettings) settings = g_settings_new ("org.gtk.Settings.Debug");
+
+          if (g_settings_get_boolean (settings, "enable-inspector-keybinding"))
+            {
+              gtk_window_set_interactive_debugging (TRUE);
+              DZL_RETURN (TRUE);
+            }
         }
 
       /*
@@ -883,13 +912,13 @@ dzl_shortcut_manager_run_fallbacks (DzlShortcutManager     *self,
                     }
 
                   if (dzl_gtk_widget_action (widget, prefix, name, param))
-                    return TRUE;
+                    DZL_RETURN (TRUE);
                 }
             }
         }
     }
 
-  return FALSE;
+  DZL_RETURN (FALSE);
 }
 
 /**
