@@ -44,6 +44,9 @@ struct _DzlSuggestionPopover
   DzlListBox         *list_box;
   GtkBox             *top_box;
 
+  /* Used for listbox workaround with selection ordering */
+  DzlSuggestionRow   *tmp_selected_row;
+
   DzlAnimation       *scroll_anim;
 
   GListModel         *model;
@@ -58,6 +61,8 @@ struct _DzlSuggestionPopover
 
   PangoEllipsizeMode  subtitle_ellipsize;
   PangoEllipsizeMode  title_ellipsize;
+
+  guint               popup_requested : 1;
 };
 
 enum {
@@ -427,7 +432,13 @@ dzl_suggestion_popover_list_box_row_selected (DzlSuggestionPopover *self,
   g_assert (!row || DZL_IS_SUGGESTION_ROW (row));
   g_assert (GTK_IS_LIST_BOX (list_box));
 
+  /* GtkListBox doesn't necessarily give us @row back when we call
+   * gtk_list_box_get_selected_row(), so this workaround allows us
+   * to continue using that API after checking for this pointer.
+   */
+  self->tmp_selected_row = DZL_SUGGESTION_ROW (row);
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_SELECTED]);
+  self->tmp_selected_row = NULL;
 }
 
 static void
@@ -696,7 +707,10 @@ dzl_suggestion_popover_popup (DzlSuggestionPopover *self)
   g_assert (DZL_IS_SUGGESTION_POPOVER (self));
 
   if (self->model == NULL || 0 == (n_items = g_list_model_get_n_items (self->model)))
-    return;
+    {
+      self->popup_requested = TRUE;
+      return;
+    }
 
   if (self->relative_to != NULL)
     {
@@ -734,6 +748,8 @@ dzl_suggestion_popover_popdown (DzlSuggestionPopover *self)
 
   g_assert (DZL_IS_SUGGESTION_POPOVER (self));
 
+  self->popup_requested = FALSE;
+
   if (!gtk_widget_get_realized (GTK_WIDGET (self)))
     return;
 
@@ -765,6 +781,13 @@ dzl_suggestion_popover_items_changed (DzlSuggestionPopover *self,
       return;
     }
 
+  if (self->popup_requested)
+    {
+      dzl_suggestion_popover_popup (self);
+      self->popup_requested = FALSE;
+      return;
+    }
+
   /*
    * If we are currently animating in the initial view of the popover,
    * then we might need to cancel that animation and rely on the elastic
@@ -774,9 +797,15 @@ dzl_suggestion_popover_items_changed (DzlSuggestionPopover *self,
       !gtk_revealer_get_child_revealed (self->revealer) &&
       (removed || added))
     {
+      g_signal_handlers_block_by_func (self->revealer,
+                                       G_CALLBACK (dzl_suggestion_popover_notify_child_revealed),
+                                       self);
       gtk_revealer_set_transition_duration (self->revealer, 0);
       gtk_revealer_set_reveal_child (self->revealer, FALSE);
       gtk_revealer_set_reveal_child (self->revealer, TRUE);
+      g_signal_handlers_unblock_by_func (self->revealer,
+                                         G_CALLBACK (dzl_suggestion_popover_notify_child_revealed),
+                                         self);
     }
 }
 
@@ -795,7 +824,7 @@ dzl_suggestion_popover_connect (DzlSuggestionPopover *self)
                              "items-changed",
                              G_CALLBACK (dzl_suggestion_popover_items_changed),
                              self,
-                             G_CONNECT_SWAPPED);
+                             G_CONNECT_AFTER | G_CONNECT_SWAPPED);
 
   if (g_list_model_get_n_items (self->model) == 0)
     {
@@ -850,8 +879,13 @@ dzl_suggestion_popover_set_model (DzlSuggestionPopover *self,
         {
           self->model = g_object_ref (model);
           dzl_suggestion_popover_connect (self);
+
+          if (self->popup_requested)
+            dzl_suggestion_popover_popup (self);
         }
 
+      self->popup_requested = FALSE;
+
       g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_MODEL]);
     }
 }
@@ -996,7 +1030,12 @@ dzl_suggestion_popover_get_selected (DzlSuggestionPopover *self)
 
   g_return_val_if_fail (DZL_IS_SUGGESTION_POPOVER (self), NULL);
 
-  row = DZL_SUGGESTION_ROW (gtk_list_box_get_selected_row (GTK_LIST_BOX (self->list_box)));
+  /* Work around row selection wonkiness in GtkListBox */
+  if (self->tmp_selected_row)
+    row = self->tmp_selected_row;
+  else
+    row = DZL_SUGGESTION_ROW (gtk_list_box_get_selected_row (GTK_LIST_BOX (self->list_box)));
+
   if (row != NULL)
     return dzl_suggestion_row_get_suggestion (row);
 
@@ -1074,3 +1113,11 @@ _dzl_suggestion_popover_adjust_margin (DzlSuggestionPopover *self,
   area->height += margin.top + margin.bottom;
 }
 
+void
+_dzl_suggestion_popover_set_click_mode (DzlSuggestionPopover *self,
+                                        gboolean              single_click)
+{
+  g_assert (DZL_IS_SUGGESTION_POPOVER (self));
+
+  gtk_list_box_set_activate_on_single_click (GTK_LIST_BOX (self->list_box), single_click);
+}
